#!/bin/bash
DEF_DIRECTORY=/opt/ImpactAcquire

# Backward compatibility check for installations prior version 3.0.0
if [ "$MVIMPACT_ACQUIRE_DATA_DIR" == "/opt/mvIMPACT_Acquire/data" ] || [[ -z "$MVIMPACT_ACQUIRE_DATA_DIR" ]]; then
    DEF_DATA_DIRECTORY="$DEF_DIRECTORY/data"
else
    DEF_DATA_DIRECTORY="$MVIMPACT_ACQUIRE_DATA_DIR"
fi

PRODUCT="ImpactAcquire"
TARNAME="ImpactAcquire"
PACKAGENAME="ImpactAcquire"
USE_DEFAULTS=NO
UNATTENDED_INSTALLATION=NO
REMOVE_ONLY=NO
MINIMAL_INSTALLATION=NO
OS_NAME="unknown"
OS_VERSION="unknown"
OS_CODENAME="unknown"
KERNEL_VERSION="unknown"
NPROC="1"
REPOSITORY_ACCESS_AVAILABLE=FALSE

TARGET_UNCAPITALIZED=undefined
GEV_SUPPORT=undefined
U3V_SUPPORT=undefined
PCIE_SUPPORT=undefined
USB2_SUPPORT=undefined
VDEV_SUPPORT=undefined
GENICAM_SUPPORT_REQUIRED=undefined
GENICAM_VERSION=3_4

ARM_ARCHITECTURE="$(uname -m)"
APT_GET_EXTRA_PARAMS=""
YUM_EXTRA_PARAMS=""
ZYPPER_EXTRA_PARAMS=""

# Define a variable for the ErrorCount and WarningCount
ERROR_NUMBER=0
WARNING_NUMBER=0

# Define variables for colorized bash output
# Foreground
red=$(tput setaf 1)
yellow=$(tput setaf 3)
green=$(tput setaf 10)
bold=$(tput bold)
# Background
reset=$(tput sgr0)

UDEVADM=$(command -v udevadm)

# Define the users real name if possible, to prevent accidental root ownership if script is invoked with sudo
if [ "$(command -v logname)" == "" ]; then
    USER=$(whoami)
else
    if [ "$(logname 2>&1 | grep -c logname:)" == "1" ]; then
        USER=$(whoami)
    else
        USER=$(logname)
    fi
fi

# If user is root, then sudo shouldn't be used
if [ "$USER" == "root" ]; then
    SUDO=
else
    SUDO=$(command -v sudo)
fi

# Check if systemd is supported or not
# Get the command of process id 1 (should be systemd on most systems)
INIT_SYSTEM=$(ps --no-headers -o comm 1)

# ---------------------------------------------------------------------------------------------------
# Functions
# ---------------------------------------------------------------------------------------------------

# Determine OS Name, version, Kernel version and number of CPU cores
function check_distro_and_version() {
    if [ -f /etc/fedora-release ]; then
        OS_NAME='Fedora'
        OS_VERSION=$(sed </etc/redhat-release s/.*release\ // | sed s/\ .*//)
    elif [ -f /etc/redhat-release ]; then
        OS_NAME='RedHat'
        OS_VERSION=$(sed </etc/redhat-release s/.*release\ // | sed s/\ .*//)
    elif [ -f /etc/centos-release ]; then
        OS_NAME='CentOS'
        OS_VERSION=$(sed </etc/centos-release s/.*release\ // | sed s/\ .*//)
    elif [ -f /etc/SuSE-release ]; then
        OS_NAME='SuSE'
        OS_VERSION=$(tr </etc/SuSE-release "\n" ' ' | sed s/.*=\ //)
    elif [ -f /etc/SUSE-brand ]; then
        OS_NAME='SuSE'
        OS_VERSION=$(tr </etc/SUSE-brand "\n" ' ' | sed s/.*=\ //)
    elif [ -f /etc/mandrake-release ]; then
        OS_NAME='Mandrake'
        OS_VERSION=$(sed </etc/mandrake-release s/.*release\ // | sed s/\ .*//)
    elif [ -x /usr/bin/lsb_release ]; then
        OS_NAME="$(lsb_release -is)" #Ubuntu
        OS_VERSION="$(lsb_release -rs)"
        OS_CODENAME="$(lsb_release -cs)"
    elif [ -f /etc/debian_version ]; then
        OS_NAME="Debian"
        OS_VERSION="$(</etc/debian_version)"
    fi
    KERNEL_VERSION=$(uname -r)
    if [ -z "$JETSON_KERNEL" ]; then
        JETSON_KERNEL=$(uname -r | grep tegra)
    fi
    # how many CPU cores available?
    if which nproc >/dev/null 2>&1; then
        NPROC=$(command nproc --all)
    else
        # sensible default
        NPROC=4
    fi
}

# Check repository access for apt-get
function checkAPTGETRepositoryAccess() {
    hosts=$(apt-cache policy | grep -Eo 'http://[^ >]+' | sed -e 's/^http:\/\///g' -e 's/\/.*//' | sort -u)
    host=($hosts)
    HOSTS_TO_BE_PINGED_COUNT=${#host[@]}
    for entry in "${host[@]}"; do
        if [[ $entry != *"microsoft"* ]] && [[ $entry != *"ppa"* ]]; then
            if ping -c2 "$entry" >/dev/null 2>&1; then
                ((PINGED_SUCCESSFULLY = PINGED_SUCCESSFULLY + 1))
            fi
        else
            ((HOSTS_TO_BE_PINGED_COUNT = HOSTS_TO_BE_PINGED_COUNT - 1))
        fi
    done
    status=$((PINGED_SUCCESSFULLY - HOSTS_TO_BE_PINGED_COUNT))
    return $status
}

# Check repository access for yum
function checkYUMRepositoryAccess() {
    hosts=$(yum repolist -v | grep Repo-baseurl | awk '{print $3}' | sed -e 's/^http:\/\///g' -e 's/^https:\/\///g' -e 's/^rsync:\/\///g' -e 's/\/.*//' | sort -u)
    host=("$hosts")
    HOSTS_TO_BE_PINGED_COUNT=${#host[@]}
    for entry in "${host[@]}"; do
        if [[ $entry != *"microsoft"* ]] && [[ $entry != *"metadata"* ]]; then
            if ping -c2 "$entry" >/dev/null 2>&1; then
                ((PINGED_SUCCESSFULLY = PINGED_SUCCESSFULLY + 1))
            fi
        else
            ((HOSTS_TO_BE_PINGED_COUNT = HOSTS_TO_BE_PINGED_COUNT - 1))
        fi
    done
    status=$((PINGED_SUCCESSFULLY - HOSTS_TO_BE_PINGED_COUNT))
    return $status
}

# Check if there is access to the package manager of the OS has access to its repositories
function checkRepositoryAccess() {
    echo
    echo "Checking repository access. This might take a moment... "
    if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ]; then
        if which apt-get >/dev/null 2>&1; then
            if checkAPTGETRepositoryAccess; then
                echo 'Success: apt-get has repository access.'
            else
                return 1
            fi
        fi
    elif [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
        if $SUDO which zypper >/dev/null 2>&1; then
            if [ "$($SUDO zypper refresh)" == 0 ]; then
                echo 'Success: zypper has repository access.'
            else
                return 1
            fi
        fi
    elif [ "$OS_NAME" == "RedHat" ] || [ "$OS_NAME" == "Fedora" ] || [ "$OS_NAME" == "CentOS" ]; then
        if $SUDO which yum >/dev/null 2>&1; then
            if checkYUMRepositoryAccess; then
                echo 'Success: yum has repository access.'
            else
                return 1
            fi
        fi
    fi
    return 0
}

# Ask for installation of a specific package
function showInputRequest() {
    printf "Do you want to install >%s< (default is 'yes')?" "$1"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
}

# Some error reporting
function couldNotInstallMessage() {
    printf "Could not install the selected package(s) >%s<. You may need to install it manually" "$1"
    ((WARNING_NUMBER = WARNING_NUMBER + 1))
}

# Some reporting about a missing package
function couldNotFindPkgManagerMessage() {
    printf "Could not find the package manager of the system; please install >%s< manually." "$1"
    ((WARNING_NUMBER = WARNING_NUMBER + 1))
}

function distroNotSupportedMessage() {
    printf "Your Linux distribution is currently not officially supported by this installation script. Please install required package >%s< manually." "$1"
    ((WARNING_NUMBER = WARNING_NUMBER + 1))
}

# Install packages for all supported distributions
function installPackages() {
    # returns 0 on success
    pkgs=("$@")
    # package manager extra parameters
    APT_GET_EXTRA_PARAMS="-y"
    YUM_EXTRA_PARAMS="-y"
    ZYPPER_EXTRA_PARAMS="--non-interactive"

    echo "Installing ${pkgs[*]}..."
    if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ] || [ "$OS_NAME" == "Raspbian" ]; then
        if which apt-get >/dev/null 2>&1; then
            $SUDO apt-get "$APT_GET_EXTRA_PARAMS" -q install "${pkgs[@]}"
            INSTALLATION_RESULT=$?
            if [ "$INSTALLATION_RESULT" -gt 0 ]; then
                couldNotInstallMessage "${pkgs[@]}"
                return 1
            fi
        else
            couldNotFindPkgManagerMessage "${pkgs[@]}"
            return 1
        fi
    elif [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
        if $SUDO which zypper >/dev/null 2>&1; then
            $SUDO zypper "$ZYPPER_EXTRA_PARAMS" install "${pkgs[@]}"
            INSTALLATION_RESULT=$?
            if [ "$INSTALLATION_RESULT" != 0 ]; then
                couldNotInstallMessage "${pkgs[@]}"
                return 1
            fi
        else
            couldNotFindPkgManagerMessage "${pkgs[@]}"
            return 1
        fi
    elif [ "$OS_NAME" == "RedHat" ] || [ "$OS_NAME" == "Fedora" ] || [ "$OS_NAME" == "CentOS" ]; then
        if $SUDO which yum >/dev/null 2>&1; then
            $SUDO yum "$YUM_EXTRA_PARAMS" install "${pkgs[@]}"
            INSTALLATION_RESULT=$?
            if [ "$INSTALLATION_RESULT" != 0 ]; then
                couldNotInstallMessage "${pkgs[@]}"
                return 1
            fi
        else
            couldNotFindPkgManagerMessage "${pkgs[@]}"
            return 1
        fi
    else
        return 1
    fi
    # success
    return 0
}

# Create softlinks including some additional error checking and cleanup
function createSoftlink() {
    if [ ! -e "$1/$2" ]; then
        echo "Error: File $1/$2 does not exist, softlink cannot be created! "
        exit 1
    fi
    if [ -e "$1/$3" ] && [ -n "$1" ] && [ -n "$3" ]; then
        rm -rf "${1:?}/${3:?}" >/dev/null 2>&1
    fi
    if ! [ -L "$1/$3" ]; then
        ln -fs "$2" "$1/$3" >/dev/null 2>&1
        if ! [ -L "$1/$3" ]; then
            $SUDO ln -fs "$2" "$1/$3" >/dev/null 2>&1
            if ! [ -L "$1/$3" ]; then
                echo "Error: Could not create softlink $1/$3, even with sudo!"
                exit 1
            fi
        fi
    fi
}

# Removing of previous installations
function remove_previous_installation() {
    $SUDO rm -f /usr/bin/DeviceConfigure >/dev/null 2>&1
    $SUDO rm -f /usr/bin/IPConfigure >/dev/null 2>&1
    $SUDO rm -f /usr/bin/ImpactControlCenter >/dev/null 2>&1
    # Delete possible remains of older installations before version 3.0.0
    $SUDO rm -f /usr/bin/mvDeviceConfigure >/dev/null 2>&1
    $SUDO rm -f /usr/bin/mvIPConfigure >/dev/null 2>&1
    $SUDO rm -f /usr/bin/wxPropView >/dev/null 2>&1
    $SUDO rm -f /etc/ld.so.conf.d/acquire.conf >/dev/null 2>&1
    $SUDO rm -f /etc/ld.so.conf.d/genicam.conf >/dev/null 2>&1
    $SUDO rm -f /etc/profile.d/acquire.sh >/dev/null 2>&1
    $SUDO rm -f /etc/profile.d/genicam.sh >/dev/null 2>&1
    $SUDO rm -f /etc/udev/rules.d/51-mvbf.rules >/dev/null 2>&1
    $SUDO rm -f /etc/udev/rules.d/52-U3V.rules >/dev/null 2>&1
    $SUDO rm -f /etc/udev/rules.d/52-mvbf3.rules >/dev/null 2>&1
    $SUDO rm -f /etc/sysctl.d/62-buffers-performance.conf >/dev/null 2>&1
    $SUDO rm -f /etc/security/limits.d/acquire.conf >/dev/null 2>&1
    # unload and delete any older version of the BN kernel module and then delete the udev rules
    BN_MODNAME="mvpci"
    MOD_STATUS=$(lsmod | grep -w $BN_MODNAME)
    if [ "x${MOD_STATUS}" != "x" ]; then
        $SUDO rmmod $BN_MODNAME
    fi
    $SUDO rm -f /lib/modules/"$(uname -r)"/kernel/misc/${BN_MODNAME}.ko
    $SUDO rm -f /etc/udev/rules.d/51-udev-pcie.rules >/dev/null 2>&1
    $SUDO rm -f /usr/local/bin/make_device.sh 2>&1
    $SUDO rm -f /usr/local/bin/device_namer.sh 2>&1
    $SUDO rm -rf /opt/mvIMPACT_Acquire/data >/dev/null 2>&1
    $SUDO rm -rf "$1" >/dev/null 2>&1
    return $?
}

# Install g++ if required
function install_gpp() {
    # Check if we have g++
    if ! which g++ >/dev/null 2>&1; then
        ADDITIONAL_PACKAGE_TO_BE_INSTALLED="g++"
        echo
        echo "A native g++ compiler is required to build the sample applications."
        echo "Currently no g++ compiler is detected on the system. Do you want to install it?"
        echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
        if [ "$USE_DEFAULTS" == "NO" ]; then
            read -r YES_NO
        else
            YES_NO=""
        fi
        echo
        if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
            printf "The >%s< compiler will not be installed." "$ADDITIONAL_PACKAGE_TO_BE_INSTALLED"
        else
            if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ] || [ "$OS_NAME" == "Raspbian" ]; then
                REQUIRED_PACKAGES=(g++)
                $SUDO apt-get update
                installPackages "${REQUIRED_PACKAGES[@]}"
            elif [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                REQUIRED_PACKAGES=(gcc-c++ make)
                installPackages "${REQUIRED_PACKAGES[@]}"
            elif [ "$OS_NAME" == "RedHat" ] || [ "$OS_NAME" == "Fedora" ] || [ "$OS_NAME" == "CentOS" ]; then
                REQUIRED_PACKAGES=(gcc gcc-c++)
                installPackages "${REQUIRED_PACKAGES[@]}"
            else
                distroNotSupportedMessage "${REQUIRED_PACKAGES[@]}"
            fi
        fi
    fi
}

# Install cmake if required
function install_cmake() {
    # Check if we have cmake 
    if ! which cmake >/dev/null 2>&1; then
        ADDITIONAL_PACKAGE_TO_BE_INSTALLED="cmake"
        echo
        echo "CMake is required to build the sample applications."
        echo "Currently CMake cannot be detected on the system. Do you want to install it?"
        echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
        if [ "$USE_DEFAULTS" == "NO" ]; then
            read -r YES_NO
        else
            YES_NO=""
        fi
        echo
        if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
            printf ">%s< will not be installed." "$ADDITIONAL_PACKAGE_TO_BE_INSTALLED"
        else
            if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ] || [ "$OS_NAME" == "Raspbian" ]; then
                REQUIRED_PACKAGES=(cmake)
                $SUDO apt-get update
                installPackages "${REQUIRED_PACKAGES[@]}"
            elif [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                REQUIRED_PACKAGES=(cmake)
                installPackages "${REQUIRED_PACKAGES[@]}"
            elif [ "$OS_NAME" == "RedHat" ] || [ "$OS_NAME" == "Fedora" ] || [ "$OS_NAME" == "CentOS" ]; then
                REQUIRED_PACKAGES=(cmake)
                installPackages "${REQUIRED_PACKAGES[@]}"
            else
                distroNotSupportedMessage "${REQUIRED_PACKAGES[@]}"
            fi
        fi
    fi
}

# Install wxWidgets
function install_wxWidgets() {
    # Do we want to install wxWidgets?
    if [ "$(wx-config --release 2>&1 | grep -c "^3.")" != "1" ] ||
        [ "$(wx-config --libs webview 2>&1 | grep -c "webview")" != "1" ] ||
        [ "$(wx-config --selected-config 2>&1 | grep -c "gtk3")" == "0" ]; then
        echo
        ADDITIONAL_PACKAGE_TO_BE_INSTALLED="wxWidgets"
        showInputRequest $ADDITIONAL_PACKAGE_TO_BE_INSTALLED
        echo "This is highly recommended, as without $ADDITIONAL_PACKAGE_TO_BE_INSTALLED, you cannot build ImpactControlCenter."
        echo
        if [ "$USE_DEFAULTS" == "NO" ]; then
            read -r YES_NO
        else
            YES_NO=""
        fi
        if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
            echo "Not installing $ADDITIONAL_PACKAGE_TO_BE_INSTALLED"
        else
            if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ] || [ "$OS_NAME" == "Raspbian" ]; then
                REQUIRED_PACKAGES=(libwxgtk-webview3.2-* libwxgtk3.2-dev build-essential libgtk2.0-dev)
                installPackages "${REQUIRED_PACKAGES[@]}"
                STATUS=$?
                if [ "$STATUS" -ne 0 ]; then
                    echo
                    echo "Could not install $ADDITIONAL_PACKAGE_TO_BE_INSTALLED v3.2, falling back to v3.0."
                    REQUIRED_PACKAGES=(libwxgtk-webview3.0-* libwxgtk3.0-dev* wx3.0-headers build-essential libgtk2.0-dev)
                    installPackages "${REQUIRED_PACKAGES[@]}"
                    STATUS=$?
                fi
                echo
                if [ "$STATUS" == 0 ] && [ "x$(which wx-config)" != "x" ]; then
                    echo "Necessary wxWidgets libraries successfully installed!"
                    echo
                else
                    echo "wxWidgets libraries could not be downloaded and installed automatically on this system!"
                    echo "Please either install wxWidgets libraries manually and re-run this installer script,"
                    echo "or re-run this script and choose not to build the wxWidgets GUI tools altogether!"
                    echo
                    exit 1
                fi
                if [ "$(update-alternatives --list wx-config | grep -c "gtk3")" == "1" ]; then
                    echo "Using GTK3 wxWidgets as default during this installation."
                    $SUDO update-alternatives --set wx-config "$(update-alternatives --list wx-config | grep gtk3)"
                    WXCONFIG_RESET_NEEDED="YES"
                fi
            elif [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                REQUIRED_PACKAGES=(wxWidgets-3_2-devel)
                installPackages "${REQUIRED_PACKAGES[@]}"
            elif [ "$OS_NAME" == "RedHat" ] || [ "$OS_NAME" == "Fedora" ] || [ "$OS_NAME" == "CentOS" ]; then
                REQUIRED_PACKAGES=(wxGTK3 wxGTK3-devel)
                installPackages "${REQUIRED_PACKAGES[@]}"
            else
                distroNotSupportedMessage "${REQUIRED_PACKAGES[@]}"
            fi
        fi
    fi
}

# Install FLTK
function install_FLTK() {
    # Do we want to install FLTK?
    if ! which fltk-config >/dev/null 2>&1; then
        ADDITIONAL_PACKAGE_TO_BE_INSTALLED="FLTK"
        echo
        showInputRequest $ADDITIONAL_PACKAGE_TO_BE_INSTALLED
        echo "This is only required if you want to build the 'ContinuousCaptureFLTK' sample."
        echo
        if [ "$USE_DEFAULTS" == "NO" ]; then
            read -r YES_NO
        else
            YES_NO=""
        fi
        if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
            echo "Not installing $ADDITIONAL_PACKAGE_TO_BE_INSTALLED"
        else
            if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ] || [ "$OS_NAME" == "Raspbian" ]; then
                OS_VERSION_TOKEN=("${OS_VERSION//./ }")
                # shellcheck disable=SC2086
                if [ "$OS_NAME" == "Ubuntu" ] && [[ ${OS_VERSION_TOKEN[0]} -ge 20 ]]; then
                    REQUIRED_PACKAGES=(libfltk1.3-dev)
                else
                    REQUIRED_PACKAGES=(libgl1-mesa-dev libfltk1.1-dev)
                fi
                installPackages "${REQUIRED_PACKAGES[@]}"
            elif [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                REQUIRED_PACKAGES=(Mesa-devel fltk fltk-devel)
                installPackages "${REQUIRED_PACKAGES[@]}"
            elif [ "$OS_NAME" == "RedHat" ] || [ "$OS_NAME" == "Fedora" ] || [ "$OS_NAME" == "CentOS" ]; then
                REQUIRED_PACKAGES=("fltk" "fltk-devel")
                installPackages "${REQUIRED_PACKAGES[@]}"
            else
                distroNotSupportedMessage "${REQUIRED_PACKAGES[@]}"
            fi
        fi
    fi
}

# Print out ASCII-Art Logo.
clear
echo ""
echo "     _____                           _                                          "
echo "    |_   _|                         | |                                         "
echo "      | | _ __ ___  _ __   __ _  ___| |_                                        "
echo "      | || '_ \` _ \| '_ \ / _\` |/ __| __|                                     "
echo "     _| || | | | | | |_) | (_| | (__| |_                                        "
echo "     \___|_| |_| |_| .__/ \__,_|\___|\__|                                       "
echo "                   | |                                                          "
echo "                   |_|                                                          "
echo "                          ___                  _                                "
echo "                         / _ \                (_)                               "
echo "                        / /_\ \ ___ __ _ _   _ _ _ __ ___                       "
echo "                        |  _  |/ __/ _\` | | | | | '__/ _ \                     "
echo "                        | | | | (_| (_| | |_| | | | |  __/                      "
echo "                        \_| |_/\___\__, |\__,_|_|_|  \___|                      "
echo "                                      | |                                       "
echo "                                      |_|                                       "
echo "                                                                                "
echo ""
echo "================================================================================"
sleep 1

# Analyze the command line arguments and react accordingly
FORCE_REPOSITORY_USAGE=NO
PATH_EXPECTED=NO
SHOW_HELP=NO
while [[ $# -gt 0 ]]; do
    if [ "$1" == "-f" ] || [ "$1" == "--force" ]; then
        FORCE_REPOSITORY_USAGE=YES
    elif [ "$1" == "-h" ] || [ "$1" == "--help" ]; then
        SHOW_HELP=YES
        break
    elif [[ ("$1" == "-u" || "$1" == "--unattended") && "$PATH_EXPECTED" == "NO" ]]; then
        UNATTENDED_INSTALLATION=YES
    elif [[ ("$1" == "-m" || "$1" == "--minimal") && "$PATH_EXPECTED" == "NO" ]]; then
        MINIMAL_INSTALLATION=YES
    elif [[ ("$1" == "-rt" || "$1" == "--runtime") && "$PATH_EXPECTED" == "NO" ]]; then
        RUNTIME_INSTALLATION=YES
    elif [[ ("$1" == "-r" || "$1" == "--remove") && "$PATH_EXPECTED" == "NO" ]]; then
        REMOVE_ONLY=YES
    elif [[ ("$1" == "-p" || "$1" == "--path") && "$PATH_EXPECTED" == "NO" ]]; then
        if [ "$2" == "" ]; then
            echo
            echo "WARNING: Path option used with no defined path, will use: $DEF_DIRECTORY directory"
            echo
            SHOW_HELP=YES
            break
        else
            PATH_EXPECTED=YES
        fi
    elif [ "$PATH_EXPECTED" == "YES" ]; then
        DEF_DIRECTORY=$1
        PATH_EXPECTED=NO
    elif [[ "$PATH_EXPECTED" == "NO" ]]; then
        # Figure out, if specific device features should be installed
        if [[ ("$1" == "-gev" || "$1" == "--gev_support" || "$1" == "-u3v" || "$1" == "--u3v_support" || "$1" == "-pcie" || "$1" == "--pcie_support" || "$1" == "-usb2" || "$1" == "--usb2_support" || "$1" == "-vdev" || "$1" == "--virtual_device_support") ]]; then

            if [[ ("$GEV_SUPPORT" == "undefined") ]]; then
                GEV_SUPPORT=FALSE
            fi

            if [[ ("$U3V_SUPPORT" == "undefined") ]]; then
                U3V_SUPPORT=FALSE
            fi

            if [[ ("$PCIE_SUPPORT" == "undefined") ]]; then
                PCIE_SUPPORT=FALSE
            fi

            if [[ ("$USB2_SUPPORT" == "undefined") ]]; then
                USB2_SUPPORT=FALSE
            fi

            if [[ ("$VDEV_SUPPORT" == "undefined") ]]; then
                VDEV_SUPPORT=FALSE
            fi

            if [[ ("$1" == "-gev") || ("$1" == "--gev_support") ]]; then
                GEV_SUPPORT=TRUE
            fi

            if [[ ("$1" == "-u3v") || ("$1" == "--u3v_support") ]]; then
                U3V_SUPPORT=TRUE
            fi

            if [[ ("$1" == "-pcie") || ("$1" == "--pcie_support") ]]; then
                PCIE_SUPPORT=TRUE
            fi

            if [[ ("$1" == "-usb2") || ("$1" == "--usb2_support") ]]; then
                USB2_SUPPORT=TRUE
            fi

            if [[ ("$1" == "-vdev") || ("$1" == "--virtual_device_support") ]]; then
                VDEV_SUPPORT=TRUE
            fi
        fi
    else
        echo 'Please check your syntax and try again!'
        SHOW_HELP=YES
    fi
    shift
done

if [ "$MINIMAL_INSTALLATION" == "YES" ] && [ "$UNATTENDED_INSTALLATION" == "YES" ]; then
    if [ "$U3V_SUPPORT" == "undefined" ] && [ "$GEV_SUPPORT" == "undefined" ] && [ "$PCIE_SUPPORT" == "undefined" ] && [ "$USB2_SUPPORT" == "undefined" ] && [ "$VDEV_SUPPORT" == "undefined" ]; then
        GEV_SUPPORT=TRUE
        U3V_SUPPORT=TRUE
        PCIE_SUPPORT=TRUE
        USB2_SUPPORT=TRUE
        VDEV_SUPPORT=FALSE
    fi
fi

if [ "$SHOW_HELP" == "YES" ]; then
    echo
    echo "Installation script for the '$PRODUCT' driver."
    echo
    echo "Default installation path: $DEF_DIRECTORY"
    echo "Usage:                     ./${0##*/} [OPTION] ... "
    echo "Example:                   ./${0##*/} -p /myPath -u"
    echo
    echo "Arguments:"
    echo "-f    --force                     Force the installation script to use the package manager of the"
    echo "                                  system to install required packages if the installation script"
    echo "                                  reports inaccessible repositories."
    echo "                                  e.g. a package source on local media."
    echo "-h    --help                      Display this help."
    echo "-p    --path                      Set the directory where the files shall be installed."
    echo
    echo "-gev  --gev_support               Install the GigE Vision related features."
    echo "-u3v  --u3v_support               Install the USB3 Vision related features."
    echo "-pcie --pcie_support              Install the BVS CA-BN (mvBlueNAOS) PCIe camera related features."
    echo "-usb2 --usb2_support              Install the BVS CA-MLC/BVS CA-IGC (mvBlueFOX) related features."
    echo "-vdev --virtual_device_support    Install the Virtual Device related features."
    echo
    echo "-u    --unattended                Unattended installation with default settings. By using"
    echo "                                  this parameter you explicitly accept the EULA."
    echo "-m    --minimal                   Minimal installation. No tools or samples will be built, and"
    echo "                                  no automatic configuration and/or optimizations will be done."
    echo "                                  By using this parameter you explicitly accept the EULA."
    echo "-r    --remove                    Remove the previously installed version for ALL devices, if found."
    echo
    echo "Note:"
    echo "                                  It is possible as well to combine unattended and minimal"
    echo "                                  installation."
    echo "                                  Combines the use of default settings from the 'minimal'"
    echo "                                  installation with not building tools and samples."
    echo
    exit 1
fi

if [ "$REMOVE_ONLY" == "YES" ] && [ "$MVIMPACT_ACQUIRE_DIR" = "" ]; then
    echo "Option to remove the previous installation chosen but no installation found!"
    echo "Quit!"
    exit 1
fi

if [ "$UNATTENDED_INSTALLATION" == "YES" ]; then
    echo
    echo "Unattended installation requested, no user interaction will be required and the"
    echo "default settings will be used."
    echo
    USE_DEFAULTS=YES
fi

if [ "$MINIMAL_INSTALLATION" == "YES" ]; then
    echo
    echo "Minimal/Runtime installation requested, no user interaction will be required, no tools or"
    echo "samples will be built and no automatic configurations or optimizations will be done."
    echo
    USE_DEFAULTS=YES
fi

# Get some details about the system
check_distro_and_version

# Get driver name, version, file. In case of multiple *.tgz files in the folder select the newest version.
if [ "$(find . -name "$PACKAGENAME*.tgz" -type f | wc -l)" != "0" ]; then
    TARNAME="$(find . -name "$PACKAGENAME*.tgz" -type f | cut -c3- | tail -1 | sed -e s/\\.tgz//)"
    if [ "$(echo "$TARNAME" | grep -c x86_64_ABI2)" != "0" ]; then
        TARGET_POINTER_LENGTH=64
        TARGET="x86_64"
        TARGET_UNCAPITALIZED="$TARGET"
        VERSION="$(find . -name "$PACKAGENAME*.tgz" -type f | cut -c3- | tail -1 | sed -e s/\\"$PACKAGENAME"// | sed -e s/\\-"$TARGET"// | sed -e s/\\_ABI2-// | sed -e s/\\.tgz//)"

        # We now only support installation on a 64-bit machine
        if [ "$(uname -m)" == "x86" ]; then
            echo "-----------------------------------------------------------------------------------"
            echo "${red}  ABORTING: Attempt to install 64-bit drivers on a 32-bit machine!   ${reset}"
            echo "-----------------------------------------------------------------------------------"
            echo
            exit 1
        fi
    elif [ "$(echo "$TARNAME" | grep -c x86_ABI2)" != "0" ]; then
        echo "-----------------------------------------------------------------------------------"
        echo "${red}  ABORTING: Attempt to install 32-bit drivers which are not supported by     "
        echo "        newer scripts anymore!                                             ${reset}"
        echo "-----------------------------------------------------------------------------------"
        echo
        exit 1
    else
        # The package states it is a package for ARM based platforms
        if [ "$(echo "$TARNAME" | grep -c ARMhf)" != "0" ]; then
            TARGET="ARMhf"
            TARGET_UNCAPITALIZED="armhf"
            TARGET_POINTER_LENGTH=32
        elif [ "$(echo "$TARNAME" | grep -c armv7ahf)" != "0" ]; then
            TARGET="armv7ahf"
            TARGET_UNCAPITALIZED="armv7ahf"
            TARGET_POINTER_LENGTH=32
        elif [ "$(echo "$TARNAME" | grep -c ARM64)" != "0" ]; then
            TARGET="ARM64"
            TARGET_UNCAPITALIZED="arm64"
            TARGET_POINTER_LENGTH=64
        else
            echo "-----------------------------------------------------------------------------------"
            echo "${red}  ABORTING: Could not determine target architecture from file name!  ${reset}"
            echo "In case the file has been renamed, please revert to the original name."
            echo "Terminating installation script now..."
            echo "-----------------------------------------------------------------------------------"
            echo
            exit 1
        fi
        VERSION="$(find . -name "$PACKAGENAME*.tgz" -type f | cut -c3- | tail -1 | sed -e s/\\"$PACKAGENAME"// | sed -e s/\\-"$TARGET"_gnu.*-// | sed -e s/\\.tgz//)"
    fi
    TARFILE="$(find . -name "$PACKAGENAME*.tgz" -type f | cut -c3- | tail -1)"
    ACT=ImpactAcquire-$TARGET-$VERSION
fi

# PCIe support is only available for 64 bit platforms
if [ "$TARGET_POINTER_LENGTH" == "32" ]; then
    PCIE_SUPPORT=FALSE
fi

# Get the source directory (the directory where the files for the installation are) and cd to it
# (The script file must be in the same directory as the source TGZ) !!!
if command -v dirname >/dev/null; then
    SCRIPTSOURCEDIR="$(dirname "$(realpath "$0")")"
fi
if [ "$SCRIPTSOURCEDIR" != "$PWD" ]; then
    if [ "$SCRIPTSOURCEDIR" == "" ] || [ "$SCRIPTSOURCEDIR" == "." ]; then
        SCRIPTSOURCEDIR="$PWD"
    fi
    cd "$SCRIPTSOURCEDIR" || exit
fi

# Set variables for Impact Acquire for later use
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
    ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf.d/acquire.conf
else
    ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf
fi

# Make sure the environment variables are set at the next boot as well
if grep -q '/etc/profile.d/' /etc/profile; then
    ACQUIRE_EXPORT_FILE=/etc/profile.d/acquire.sh
else
    ACQUIRE_EXPORT_FILE=/etc/profile
fi

# Set variables for GenICam for later use
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
    GENICAM_LDSOCONF_FILE=/etc/ld.so.conf.d/genicam.conf
else
    GENICAM_LDSOCONF_FILE=/etc/ld.so.conf
fi
# Make sure the GenICam related environment variables for are set at the next boot as well
if grep -q '/etc/profile.d/' /etc/profile; then
    GENICAM_EXPORT_FILE=/etc/profile.d/genicam.sh
else
    GENICAM_EXPORT_FILE=/etc/profile
fi

# A quick check whether the Version has a correct format (due to other files being in the same directory..?)
if [ "$(echo "$VERSION" | grep -c '^[0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{1,2\}')" == "0" ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${red}  ABORTING: Script could not determine a valid $PRODUCT *.tgz file!  "
    echo "${reset}-----------------------------------------------------------------------------------"
    echo "  This script could not extract a valid version number from the *.tgz file"
    echo "  This script determined $TARFILE as the file containing the installation data."
    echo "  It is recommended that only this script and the correct *.tgz file reside in this directory."
    echo "  Please remove all other files and try again."
    exit 1
fi

# A quick check whether the user has been determined
if [ "$USER" == "" ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${red}  ABORTING: Script could not determine a valid user!  ${reset}"
    echo "-----------------------------------------------------------------------------------"
    echo "  This script could not determine the user of this shell"
    echo "  Please make sure this is a valid login shell!"
    exit 1
fi

YES_NO=
# Ask whether to use the defaults or proceed with an interactive installation
if [ "$UNATTENDED_INSTALLATION" == "NO" ] && [ "$MINIMAL_INSTALLATION" == "NO" ]; then
    echo
    echo "Would you like this installation to run in unattended mode?"
    echo "Using this mode you explicitly agree to the EULA(End User License Agreement)!"
    echo "No user interaction will be required, and the default settings will be used!"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    read -r YES_NO
else
    YES_NO=""
fi
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    USE_DEFAULTS=NO
else
    USE_DEFAULTS=YES
fi

YES_NO=
# Check if package manager has access to repositories
if [ "$MINIMAL_INSTALLATION" == "NO" ] && [ "$REMOVE_ONLY" != "YES" ]; then
    if checkRepositoryAccess; then
        REPOSITORY_ACCESS_AVAILABLE="YES"
    else
        echo "-----------------------------------------------------------------------------------"
        echo "${yellow}  WARNING: There seems to be no repository access!${reset}                "
        echo "-----------------------------------------------------------------------------------"
        echo "  This script could not get access to the system's package repositories."
        echo "  If additional packages are requiered during installation e.g.: g++,"
        echo "  wxWidgets, expat, FLTK, ... some features might not be available."
        echo
        echo "  In certain cases this might be a result of specific firewall settings or local   "
        echo "  package repositores. It is possible to let the script try to use those package   "
        echo "  repositories even if the access could not be verified.                           "
        echo
        echo "  Would you like this installation to force the usage of the package manager to    "
        echo "  install some additional required packages?"
        echo "  Hit 'y' + <Enter> for 'yes', or just <Enter> for 'no'."

        if [ "$USE_DEFAULTS" == "NO" ]; then
            read -r YES_NO
        else
            YES_NO=""
        fi
        if [ "$YES_NO" == "y" ] || [ "$YES_NO" == "Y" ]; then
            FORCE_REPOSITORY_USAGE="YES"
        fi
    fi
fi

YES_NO=
# Here we will ask the user if we shall start the installation process
echo
echo "-----------------------------------------------------------------------------------"
echo "${bold}Host System:${reset}"
echo "-----------------------------------------------------------------------------------"
echo
echo "${bold}OS:                             ${reset} $OS_NAME"
echo "${bold}OS Version:                     ${reset} $OS_VERSION"
echo "${bold}OS Codename:                    ${reset} $OS_CODENAME"
echo "${bold}Kernel:                         ${reset} $KERNEL_VERSION"
echo "${bold}Platform:                       ${reset} $TARGET"
echo
if [ "$REMOVE_ONLY" == "YES" ]; then
    echo "${bold}Installation directory to be removed: ${reset} $MVIMPACT_ACQUIRE_DIR"
else
    echo "-----------------------------------------------------------------------------------"
    echo "${bold}Configuration:${reset}"
    echo "-----------------------------------------------------------------------------------"
    echo
    echo "${bold}Installation for user:            ${reset} $USER"
    echo "${bold}Installation directory:           ${reset} $DEF_DIRECTORY"
    echo "${bold}Data directory:                   ${reset} $DEF_DATA_DIRECTORY"
    echo "${bold}Source directory:                 ${reset} $(echo "$SCRIPTSOURCEDIR" | sed -e 's/\/\.//')"
    echo "${bold}Version:                          ${reset} $VERSION"
    echo "${bold}TAR-File:                         ${reset} $TARFILE"
    echo
    echo "${bold}ldconfig:"
    echo "${bold}GenICam:                        ${reset} $GENICAM_LDSOCONF_FILE"
    echo "${bold}Impact Acquire:                 ${reset} $ACQUIRE_LDSOCONF_FILE"
    echo
    echo "${bold}Exports:"
    echo "${bold}GenICam:                        ${reset} $GENICAM_EXPORT_FILE"
    echo "${bold}Impact Acquire:                 ${reset} $ACQUIRE_EXPORT_FILE"
fi
echo

echo
echo "-----------------------------------------------------------------------------------"
echo
echo "Do you want to continue (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$USE_DEFAULTS" == "NO" ]; then
    read -r YES_NO
else
    YES_NO=""
fi
echo

# If the user is choosing no, we will abort the installation, else we will start the process.
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    echo "Quit!"
    exit
fi

if [ "$REMOVE_ONLY" == "YES" ]; then
    remove_previous_installation "$MVIMPACT_ACQUIRE_DIR"
    result=$?
    if [ $result == 0 ]; then
        echo "Previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR) removed successfully!"
    else
        echo "Error removing previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR)!"
    fi
    exit "$result"
fi

# End User License Agreement
YES_NO="r"
while [ "$YES_NO" == "r" ] || [ "$YES_NO" == "R" ]; do
    echo
    echo "Do you accept the End User License Agreement (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', 'r' + <Enter> to read the EULA or "
    echo "just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
        if [ "$YES_NO" == "r" ] || [ "$YES_NO" == "R" ]; then
            if [ "x$(command -v more)" != "x" ]; then
                EULA_SHOW_COMMAND="more -d"
            else
                EULA_SHOW_COMMAND="cat"
            fi
            tar -xzf "$SCRIPTSOURCEDIR/$TARFILE" -C /tmp ImpactAcquire-"$TARGET"-"$VERSION"/doc/EULA.txt --strip-components=2 && $EULA_SHOW_COMMAND /tmp/EULA.txt && rm /tmp/EULA.txt && sleep 1
            # clear the stdin buffer in case user spammed the Enter key
            while read -r -t 0; do read -r; done
        fi
    else
        YES_NO=""
    fi
done

# If the user is choosing no, we will abort the installation, else we continue.
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    echo "Quit!"
    exit
fi

echo
echo "-----------------------------------------------------------------------------------"
echo "${bold}BY INSTALLING THIS SOFTWARE YOU HAVE AGREED TO THE EULA(END USER LICENSE AGREEMENT)${reset}"
echo "-----------------------------------------------------------------------------------"
echo

# First of all ask whether to dispose of the old Impact Acquire installation
if [[ "$MVIMPACT_ACQUIRE_DIR" != "" ]] && [[ -d "$MVIMPACT_ACQUIRE_DIR" ]]; then
    DEFAULT_KEEP="yes"
    INCOMPATIBLE_VERSION="no"
    NEW_MAJOR=$(echo "$VERSION" | cut -d . -f 1)
    # Determine the version number of the previous installation
    OLD_DEVICE_MANAGER="$(basename "$(find "$MVIMPACT_ACQUIRE_DIR"/lib/"${TARGET_UNCAPITALIZED}" -name "libmvDeviceManager.so.*.*.*" -print -quit)")"
    if [ "$OLD_DEVICE_MANAGER" != "" ]; then
        OLD_MAJOR=$(echo "$OLD_DEVICE_MANAGER" | cut -d . -f 3)
        OLD_MINOR=$(echo "$OLD_DEVICE_MANAGER" | cut -d . -f 4)
        OLD_RELEASE=$(echo "$OLD_DEVICE_MANAGER" | exec cut -d . -f 5)
        echo "Previous installation version $OLD_MAJOR.$OLD_MINOR.$OLD_RELEASE detected at: $MVIMPACT_ACQUIRE_DIR"
        if [ "$OLD_MAJOR" != "$NEW_MAJOR" ]; then
            INCOMPATIBLE_VERSION="yes"
            DEFAULT_KEEP="no"
            echo "This version is incompatible with the new installation and you must remove it!"
        fi
    fi
    echo "Do you want to keep the previous installation (default is '${DEFAULT_KEEP}')?"
    echo "If you select no, Impact Acquire will be removed for ALL installed products!"
    if [ "$INCOMPATIBLE_VERSION" == "yes" ]; then
        echo "Hit 'y' + <Enter> for 'yes', or just <Enter> for 'no'."
    else
        echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    fi
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=$DEFAULT_KEEP
    fi
    echo
    YES_NO=$(echo "$YES_NO" | tr '[:upper:]' '[:lower:]')
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "no" ]; then
        remove_previous_installation "$MVIMPACT_ACQUIRE_DIR"
        result=$?
        if [ $result == 0 ]; then
            echo "Previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR) removed successfully!"
        else
            echo "Error removing previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR)!"
            echo "$result"
        fi
        if [ "$INIT_SYSTEM" == "systemd" ] && [ -f "/etc/systemd/system/resize_usbfs_buffersize.service" ]; then
            $SUDO systemctl stop resize_usbfs_buffersize.service
            $SUDO systemctl disable resize_usbfs_buffersize.service
            $SUDO rm -f /etc/systemd/system/resize_usbfs_buffersize.service
            result=$?
            if [ $result == 0 ]; then
                echo "usbcore.usbfs_memory_mb systemd service (/etc/systemd/system/resize_usbfs_buffersize.service) disabled and removed successfully!"
            else
                echo "Error removing usbcore.usbfs_memory_mb systemd service (/etc/systemd/system/resize_usbfs_buffersize.service)!"
                echo "$result"
            fi
        fi
    else
        echo "Previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR) NOT removed!"
        if [ "$INCOMPATIBLE_VERSION" == "yes" ]; then
            echo "Stopping the installer due to incompatible previous installation version $OLD_MAJOR.$OLD_MINOR.$OLD_RELEASE!"
            exit 1
        fi
    fi
fi
# Determine whether GEV, U3V and/or PCIe device types shall be supported on this system
echo ""
echo "Should Impact Acquire support GEV devices, such as BVS CA-GT1, CA-GX0 or CA-GX2 (mvBlueCOUGAR) (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."

if [ "$GEV_SUPPORT" == "undefined" ]; then
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        GEV_SUPPORT=FALSE
    else
        GEV_SUPPORT=TRUE
    fi
else
    echo GEV_SUPPORT="$GEV_SUPPORT"
fi
echo ""
echo "Should U3V device support, such as BVS CA-SF (mvBlueFOX3) be installed (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$U3V_SUPPORT" == "undefined" ]; then
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        U3V_SUPPORT=FALSE
    else
        U3V_SUPPORT=TRUE
    fi
else
    echo U3V_SUPPORT="$U3V_SUPPORT"
fi
echo ""
echo "Should PCIe device support be installed (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$PCIE_SUPPORT" == "undefined" ]; then
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        PCIE_SUPPORT=FALSE
    else
        PCIE_SUPPORT=TRUE
    fi
else
    echo PCIE_SUPPORT="$PCIE_SUPPORT"
fi

# Check if there is at least one feature selected which requires GenICam support
if [ "$U3V_SUPPORT" == "TRUE" ] || [ "$GEV_SUPPORT" == "TRUE" ] || [ "$PCIE_SUPPORT" == "TRUE" ]; then
    GENICAM_SUPPORT_REQUIRED=TRUE
fi

echo ""
echo "Should Impact Acquire support USB2 devices (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$USB2_SUPPORT" == "undefined" ]; then
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        USB2_SUPPORT=FALSE
    else
        USB2_SUPPORT=TRUE
    fi
else
    echo USB2_SUPPORT="$USB2_SUPPORT"
fi

echo ""
echo "Should support for the Virtual Device be installed (default is 'no')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$VDEV_SUPPORT" == "undefined" ]; then
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
        VDEV_SUPPORT=FALSE
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        VDEV_SUPPORT=FALSE
    else
        VDEV_SUPPORT=TRUE
    fi
else
    echo VDEV_SUPPORT="$VDEV_SUPPORT"
fi

if [ "$U3V_SUPPORT" == "FALSE" ] && [ "$GEV_SUPPORT" == "FALSE" ] && [ "$PCIE_SUPPORT" == "FALSE" ] && [ "$USB2_SUPPORT" == "FALSE" ] && [ "$VDEV_SUPPORT" == "FALSE" ]; then
    echo "Warning: As none of the supported technologies provided by this package has been selected"
    echo "for installation. This means no device will be accessible after the installation has"
    echo "finished."
    ((ERROR_NUMBER = ERROR_NUMBER + 1))
fi

# Create the acquire.conf file if the system is supporting ld.so.conf.d
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
    $SUDO rm -f $ACQUIRE_LDSOCONF_FILE
    $SUDO touch $ACQUIRE_LDSOCONF_FILE
    $SUDO chmod a+r $ACQUIRE_LDSOCONF_FILE # in case umask has changed the file permissions
fi

# Create the acquire export file if the system is supporting profile.d
if grep -q '/etc/profile.d/' /etc/profile; then
    $SUDO rm -f $ACQUIRE_EXPORT_FILE
    $SUDO touch $ACQUIRE_EXPORT_FILE
    $SUDO chmod a+r $ACQUIRE_EXPORT_FILE # in case umask has changed the file permissions
fi

if [ "$GENICAM_SUPPORT_REQUIRED" == "TRUE" ]; then
    # Create the genicam.conf file if the system is supporting ld.so.conf.d
    if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
        $SUDO rm -f $GENICAM_LDSOCONF_FILE
        $SUDO touch $GENICAM_LDSOCONF_FILE
        $SUDO chmod a+r $GENICAM_LDSOCONF_FILE # in case umask has changed the file permissions
    fi

    # Create the genicam export file if the system is supporting profile.d
    if grep -q '/etc/profile.d/' /etc/profile; then
        $SUDO rm -f $GENICAM_EXPORT_FILE
        $SUDO touch $GENICAM_EXPORT_FILE
        $SUDO chmod a+r $GENICAM_EXPORT_FILE # in case umask has changed the file permissions
    fi
fi

# Check if the destination directory exist, else create it
if ! [ -d "$DEF_DIRECTORY" ]; then
    # the destination directory does not yet exist
    # first try to create it as a normal user
    mkdir -p "$DEF_DIRECTORY" >/dev/null 2>&1
    if ! [ -d "$DEF_DIRECTORY" ]; then
        # that didn't work
        # now try it as superuser
        $SUDO mkdir -p "$DEF_DIRECTORY"
        STATUS=$?
    fi
    if ! [ -d "$DEF_DIRECTORY" ]; then
        echo "ERROR: Could not create target directory '$DEF_DIRECTORY'."
        echo "Problem: $STATUS"
        echo "Maybe you specified a partition that was mounted read only?"
        echo
        exit 1
    fi
else
    echo "Installation directory already exists."
fi

# in case the directory already existed BUT it belongs to other user
$SUDO chown -R "$USER": "$DEF_DIRECTORY"

# Check the actual tarfile
if ! [ -r "$TARFILE" ]; then
    echo "ERROR: could not read '$TARFILE'."
    echo
    exit 1
fi

# Now unpack the tar-file into the target directory
cd /tmp || exit 1

if [[ "$TARGET" == *"ARM"* ]]; then
    rm -rf ImpactAcquire-ARM*
elif [[ "$TARGET" == "x86_64" ]]; then
    rm -rf ImpactAcquire-x86_64
fi

### TODO ACQ-4872 Extracting the packages differs slightly since ancient times so this has to be unified

tar xfz "$SCRIPTSOURCEDIR/$TARFILE"
cd "$ACT" || exit 1
cp -R -d ./* "$DEF_DIRECTORY"

if [ "$GENICAM_SUPPORT_REQUIRED" == "TRUE" ]; then
    if ! [ -r "$GENICAM_EXPORT_FILE" ]; then
        echo "Error : cannot write to '$GENICAM_EXPORT_FILE'."
        echo "After the next boot, the required environment variables will not be set."
        ((ERROR_NUMBER = ERROR_NUMBER + 1))
        echo
    else
        # tests below do not yet check for *commented out* export lines
        if grep -q "GENICAM_ROOT=" "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_ROOT already defined in $GENICAM_EXPORT_FILE."
        else
            $SUDO sh -c "echo 'export GENICAM_ROOT=$DEF_DIRECTORY/runtime' >> $GENICAM_EXPORT_FILE"
        fi
        if grep -q "GENICAM_ROOT_V$GENICAM_VERSION=" "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_ROOT_V$GENICAM_VERSION already defined in $GENICAM_EXPORT_FILE."
        else
            $SUDO sh -c "echo 'export GENICAM_ROOT_V$GENICAM_VERSION=$DEF_DIRECTORY/runtime' >> $GENICAM_EXPORT_FILE"
        fi

        if grep -q "GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH=" "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH already defined in $GENICAM_EXPORT_FILE."
        else
            $SUDO sh -c "echo 'if [ x\$GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH == x ]; then
   export GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH=$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED
elif [ x\$GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH != x$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED ]; then
   if ! \$(echo \$GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH | grep -q \":$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED\"); then
      export GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH=\$GENICAM_GENTL'$TARGET_POINTER_LENGTH'_PATH:$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED
   fi
fi' >> $GENICAM_EXPORT_FILE"
        fi

        # Since Impact Acquire version 2.7.0, version 2.4 of the GenICam cache should be able to coexist with
        # version 2.3 however they must point to different folders!
        # Since Impact Acquire version 2.14.0, version 3.0 of the GenICam cache should be able to coexist with
        # version 2.3 and 2.4 however they must point to different folders!
        # Since Impact Acquire version 2.28.0, version 3.1.0 of the GenICam cache should be able to coexist with
        # version 2.3, 2.4 and 3.0 however they must point to different folders!
        # Since Impact Acquire version 2.45.0, version 3.3.0 of the GenICam cache should be able to coexist with
        # version 2.3, 2.4, 3.0 and 3.1 however they must point to different folders!
        # Since Impact Acquire version 2.50.0, version 3.4.0 of the GenICam cache should be able to coexist with
        # version 2.3, 2.4, 3.0, 3.1 and 3.3.0 however they must point to different folders!
        if grep -q 'GENICAM_CACHE_V2_3=' "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_CACHE_V2_3 already defined in $GENICAM_EXPORT_FILE."
        fi
        if grep -q 'GENICAM_CACHE_V2_4=' "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_CACHE_V2_4 already defined in $GENICAM_EXPORT_FILE."
        fi
        if grep -q 'GENICAM_CACHE_V3_0=' "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_CACHE_V3_0 already defined in $GENICAM_EXPORT_FILE."
        fi
        if grep -q 'GENICAM_CACHE_V3_1=' "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_CACHE_V3_1 already defined in $GENICAM_EXPORT_FILE."
        fi
        if grep -q 'GENICAM_CACHE_V3_3=' "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_CACHE_V3_3 already defined in $GENICAM_EXPORT_FILE."
        fi
        if grep -q 'GENICAM_CACHE_V3_4=' "$GENICAM_EXPORT_FILE"; then
            echo "GENICAM_CACHE_V3_4 already defined in $GENICAM_EXPORT_FILE."
        else
            $SUDO mkdir -p "$DEF_DIRECTORY/runtime/cache/v$GENICAM_VERSION"
            $SUDO chmod -R 777 "$DEF_DIRECTORY/runtime/cache"
            $SUDO sh -c "echo 'export GENICAM_CACHE_V3_4='$DEF_DIRECTORY'/runtime/cache/v3_4' >> $GENICAM_EXPORT_FILE"
        fi
        if grep -q 'GENICAM_LOG_CONFIG_V'$GENICAM_VERSION'=' $GENICAM_EXPORT_FILE; then
            echo 'GENICAM_LOG_CONFIG_V'$GENICAM_VERSION' already defined in' $GENICAM_EXPORT_FILE.
        else
            $SUDO sh -c "echo 'export GENICAM_LOG_CONFIG_V'$GENICAM_VERSION'=$DEF_DIRECTORY/runtime/log/config-unix/DefaultLogging.properties' >> $GENICAM_EXPORT_FILE"
        fi
    fi
fi

# Set the necessary exports and library paths
cd "$DEF_DIRECTORY" || exit 1
if grep -q 'MVIMPACT_ACQUIRE_DIR=' "$ACQUIRE_EXPORT_FILE"; then
    echo 'MVIMPACT_ACQUIRE_DIR already defined in' "$ACQUIRE_EXPORT_FILE".
else
    $SUDO sh -c "echo 'export MVIMPACT_ACQUIRE_DIR=$DEF_DIRECTORY' >> $ACQUIRE_EXPORT_FILE"
fi

if grep -q "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED" $ACQUIRE_LDSOCONF_FILE; then
    echo "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED already defined in" $ACQUIRE_LDSOCONF_FILE.
else
    $SUDO sh -c "echo '$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED' >> $ACQUIRE_LDSOCONF_FILE"
fi

if [ "$TARGET" != "armv7ahf" ]; then
    # not installing libexpat for the above platforms
    if grep -q "$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib" $ACQUIRE_LDSOCONF_FILE; then
        echo "$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib already defined in" $ACQUIRE_LDSOCONF_FILE.
    else
        $SUDO sh -c "echo '$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib' >> $ACQUIRE_LDSOCONF_FILE"
    fi
fi

# Now do the shared linker setup
if [ "$GENICAM_SUPPORT_REQUIRED" == "TRUE" ]; then
    if ! [ -r "$GENICAM_LDSOCONF_FILE" ]; then
        echo "Error : cannot write to '$GENICAM_LDSOCONF_FILE'."
        echo 'Execution will fail, as at run time, the shared objects will not be found.'
        ((ERROR_NUMBER = ERROR_NUMBER + 1))
        echo
    else
        if [ "$TARGET" = "ARMhf" ] || [ "$TARGET" = "armv7ahf" ]; then
            GENILIBPATH=Linux32_ARMhf
        elif [ "$TARGET" = "ARM64" ]; then
            GENILIBPATH=Linux64_ARM
        elif [ "$TARGET" = "x86_64" ]; then
            GENILIBPATH=Linux64_x64
        fi
        # tests below do not check for *commented out* link lines
        # must later add sub-string check

        # acquire libs
        if grep -q "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED" "$GENICAM_LDSOCONF_FILE"; then
            echo "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED already defined in" "$GENICAM_LDSOCONF_FILE".
        else
            $SUDO sh -c "echo '$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED' >> $GENICAM_LDSOCONF_FILE"
        fi

        # GenICam libs
        if grep -q "$DEF_DIRECTORY/runtime/bin/$GENILIBPATH" "$GENICAM_LDSOCONF_FILE"; then
            echo "$DEF_DIRECTORY/runtime/bin/$GENILIBPATH already defined in" "$GENICAM_LDSOCONF_FILE".
        else
            $SUDO sh -c "echo '$DEF_DIRECTORY/runtime/bin/$GENILIBPATH' >> $GENICAM_LDSOCONF_FILE"
        fi
    fi
fi

# This variable must be exported, or else ImpactControlCenter-related make problems can arise
export MVIMPACT_ACQUIRE_DIR=$DEF_DIRECTORY

# Clean up /tmp
rm -rf "/tmp/$ACT" "/tmp/ImpactAcquire-$VERSION"

#create softlinks for the Toolkits libraries
if [ "$TARGET" != "armv7ahf" ]; then
    # not installing for the above platforms
    createSoftlink "$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib" "$(ls "$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib" | grep "libexpat.so.[[:digit:]]*.[[:digit:]]*.[[:digit:]]*")" libexpat.so.1
    createSoftlink "$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib" libexpat.so.1 libexpat.so
    createSoftlink "$DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage/$TARGET_UNCAPITALIZED" "$(ls "$DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage/$TARGET_UNCAPITALIZED" | grep "libfreeimage-3.[[:digit:]]*.[[:digit:]]*.so")" libfreeimage.so.3
    createSoftlink "$DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage/$TARGET_UNCAPITALIZED" libfreeimage.so.3 libfreeimage.so
    if [ "$U3V_SUPPORT" == "TRUE" ]; then
        createSoftlink "$DEF_DIRECTORY/Toolkits/libusb-1.0.21/bin/$TARGET_UNCAPITALIZED/lib" libusb-1.0.so.0.1.0 libusb-1.0.so.0
        createSoftlink "$DEF_DIRECTORY/Toolkits/libusb-1.0.21/bin/$TARGET_UNCAPITALIZED/lib" libusb-1.0.so.0 libusb-1.0.so
        createSoftlink "$DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib" "$(ls "$DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib" | grep "libudev\.so.[[:digit:]].[[:digit:]]*.[[:digit:]]*")" libudev.so.1
        createSoftlink "$DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib" libudev.so.1 libudev.so
    else
        $SUDO rm -rf "$DEF_DIRECTORY/Toolkits/libusb-1.0.21" >/dev/null 2>&1
        $SUDO rm -rf "$DEF_DIRECTORY/Toolkits/libudev" >/dev/null 2>&1
    fi
fi

# An important distinction has to be made here between 32bit and 64bit ARM systems
if [ "$TARGET" != "ARM64" ] && [ "$TARGET" != "x86_64" ]; then
    # Since the native make target for 32bit ARM architectures can have many different values
    # (eg. armv7l, arm7ahf etc. ) and will not be 'armhf', a softlink has to be created,
    # otherwise the our apps and tools will not be able to be linked with the our libraries.
    if [ "$TARGET" != "armv7ahf" ]; then
        # not installing for the above platforms
        createSoftlink "$DEF_DIRECTORY/lib" "$TARGET_UNCAPITALIZED" "$ARM_ARCHITECTURE"
        createSoftlink "$DEF_DIRECTORY/Toolkits/expat/bin" "$TARGET_UNCAPITALIZED" "$ARM_ARCHITECTURE"
        createSoftlink "$DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage" "$TARGET_UNCAPITALIZED" "$ARM_ARCHITECTURE"
        if [ "$U3V_SUPPORT" == "TRUE" ]; then
            createSoftlink "$DEF_DIRECTORY/Toolkits/libudev/bin" "$TARGET_UNCAPITALIZED" "$ARM_ARCHITECTURE"
            createSoftlink "$DEF_DIRECTORY/Toolkits/libusb-1.0.21/bin" "$TARGET_UNCAPITALIZED" "$ARM_ARCHITECTURE"
        fi
    fi
else
    # In case of 64bit ARM architectures, they always report 'aarch64' back, which makes our lives
    # considerably easier. In this case we do not need to create softlinks, but the $ARM_ARCHITECTURE
    # needs to be overwritten so that the mvApps softlinks in /usr/bin will point to the correct path.
    ARM_ARCHITECTURE="$TARGET_UNCAPITALIZED"
fi

# Update the library cache with ldconfig
$SUDO /sbin/ldconfig

if [ "$MINIMAL_INSTALLATION" == "NO" ] || [ "$RUNTIME_INSTALLATION" == "YES" ]; then
    if { [ "$REPOSITORY_ACCESS_AVAILABLE" == "YES" ] || [ "$FORCE_REPOSITORY_USAGE" == "YES" ]; } || [ "$USE_DEFAULTS" == "YES" ]; then
        ADDITIONAL_PACKAGE_TO_BE_INSTALLED=""

        # Install needed libraries and compiler

        install_gpp

        install_cmake

        install_wxWidgets

        install_FLTK

        # In case GEV devices should not be supported remove IPConfigure
        if [ "$GEV_SUPPORT" == "FALSE" ]; then
            if [ -d "$DEF_DIRECTORY/apps/IPConfigure" ]; then
                $SUDO rm -rf "$DEF_DIRECTORY/apps/IPConfigure" >/dev/null 2>&1
            fi
        fi

        # In case we are compiling on a SuSE flavor, remove DeviceConfigure because wxWidgets is no longer compatible
        if [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
            if [ -d "$DEF_DIRECTORY/apps/DeviceConfigure" ]; then
                $SUDO rm -rf "$DEF_DIRECTORY/apps/DeviceConfigure" >/dev/null 2>&1
            fi
            if [ "$MINIMAL_INSTALLATION" == "NO" ]; then
                echo "The DeviceConfigure application will not be built because your distribution does not supply all the necessary wxWidget libraries."
            fi
        fi

        if [ "$MINIMAL_INSTALLATION" == "NO" ]; then
            echo
            echo "Do you want the tools and samples to be built (default is 'yes')?"
            echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
            if [ "$USE_DEFAULTS" == "NO" ]; then
                read -r YES_NO
            else
                YES_NO=""
            fi
            if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                echo
                echo "The tools and samples will not be built."
                echo "To build them explicitly use CMake with the file '$DEF_DIRECTORY/apps/CMakeLists.txt'."
            else
                cd "$DEF_DIRECTORY" || exit 1
                $SUDO chown -R "$USER": "$DEF_DIRECTORY"

                # build tools and applications using CMake
                CMAKE_BUILD_OK=FALSE
                CMAKE_BUILD_DIRECTORY="$DEF_DIRECTORY/apps/build.apps"
                mkdir -p "$CMAKE_BUILD_DIRECTORY"
                $SUDO chmod -R 777 "$CMAKE_BUILD_DIRECTORY"
                cd "$CMAKE_BUILD_DIRECTORY" || exit 1
                if which cmake >/dev/null 2>&1; then
                    if cmake -G "Unix Makefiles" "$DEF_DIRECTORY/apps"; then
                        echo "Building the tools and samples using $NPROC cores."
                        if ! make all "-j$NPROC"; then
                            echo
                            echo "Problems were detected when building the tools and samples."
                            echo "Some applications may not have been built correctly."
                            ((WARNING_NUMBER = WARNING_NUMBER + 1))
                        fi
                        CMAKE_BUILD_OK=TRUE
                    else
                        echo
                        echo "The tools and samples cannot be built!"
                        echo "There were problems using CMake."
                        ((WARNING_NUMBER = WARNING_NUMBER + 1))
                    fi
                else
                    echo
                    echo "The tools and samples cannot be built!"
                    echo "CMake could not be found on the system."
                    ((WARNING_NUMBER = WARNING_NUMBER + 1))
                fi
                cd "$DEF_DIRECTORY" || exit 1

                # Copy the built applications back to their source directories
                if [ "$CMAKE_BUILD_OK" == "TRUE" ]; then
                    BIN_DIRECTORY="$CMAKE_BUILD_DIRECTORY/bin"
                    for BIN_FILE in "$BIN_DIRECTORY"/*; do
                        SOURCE_CODE_DIR="$DEF_DIRECTORY/apps/"$(basename "$BIN_FILE") # assumes source directory name is the same as the application name
                        if [ -x "$BIN_FILE" ] && [ -d "$SOURCE_CODE_DIR" ]; then
                            DESTINATION_DIR="$SOURCE_CODE_DIR/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')"
                            mkdir -p "$DESTINATION_DIR"
                            $SUDO chmod -R 777 "$DESTINATION_DIR"
                            if ! cp "$BIN_FILE" "$DESTINATION_DIR"; then
                                echo
                                echo "Problem copying '$(basename "$BIN_FILE")' to '$DESTINATION_DIR'"
                                ((WARNING_NUMBER = WARNING_NUMBER + 1))
                            else
                                $SUDO chown -R "$USER": "$DESTINATION_DIR"
                            fi
                        fi
                    done
                fi
                rm -rf "$DEF_DIRECTORY/apps/build.apps"
                # make sure permissions are still OK
                $SUDO chown -R "$USER": "$DEF_DIRECTORY"

                # Should our tools be linked in /usr/bin?
                if [ "$GEV_SUPPORT" == "TRUE" ]; then
                    if [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                        GUI_APPS="ImpactControlCenter and IPConfigure"
                    else
                        GUI_APPS="ImpactControlCenter, IPConfigure and DeviceConfigure"
                    fi
                else
                    if [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                        GUI_APPS="ImpactControlCenter and IPConfigure"
                    else
                        GUI_APPS="ImpactControlCenter and DeviceConfigure"
                    fi
                fi
                if [ "$CMAKE_BUILD_OK" == "TRUE" ]; then
                    echo "Do you want to set a link to /usr/bin for $GUI_APPS (default is 'yes')?"
                    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
                    if [ "$USE_DEFAULTS" == "NO" ]; then
                        read -r YES_NO
                    else
                        YES_NO=""
                    fi
                    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                        echo "Will not set any new link to /usr/bin."
                    else
                        if [ -r /usr/bin ]; then
                            # Set ImpactControlCenter
                            if [ -r "$DEF_DIRECTORY/apps/ImpactControlCenter/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/ImpactControlCenter" ]; then
                                $SUDO rm -f /usr/bin/ImpactControlCenter
                                $SUDO ln -sf "$DEF_DIRECTORY/apps/ImpactControlCenter/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/ImpactControlCenter" /usr/bin/ImpactControlCenter
                            fi
                            # Set IPConfigure
                            if [ "$GEV_SUPPORT" == "TRUE" ]; then
                                if [ -r "$DEF_DIRECTORY/apps/IPConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/IPConfigure" ]; then
                                    $SUDO rm -f /usr/bin/IPConfigure
                                    $SUDO ln -sf "$DEF_DIRECTORY/apps/IPConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/IPConfigure" /usr/bin/IPConfigure
                                    $SUDO ln -sf "$DEF_DIRECTORY/apps/IPConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/IPConfigure" /usr/bin/mvIPConfigure # This used to be the name of this application for versions smaller than 3.0.0
                                fi
                            fi
                            # Set DeviceConfigure
                            if [ -r "$DEF_DIRECTORY/apps/DeviceConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/DeviceConfigure" ]; then
                                $SUDO rm -f /usr/bin/DeviceConfigure
                                $SUDO ln -sf "$DEF_DIRECTORY/apps/DeviceConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/DeviceConfigure" /usr/bin/DeviceConfigure
                                $SUDO ln -sf "$DEF_DIRECTORY/apps/DeviceConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/DeviceConfigure" /usr/bin/mvDeviceConfigure # This used to be the name of this application for versions smaller than 3.0.0
                            fi
                        fi
                    fi
                fi
            fi

            if [ "$CMAKE_BUILD_OK" == "TRUE" ]; then
                # Should ImpactControlCenter check weekly for updates?
                echo "Do you want ImpactControlCenter to check for updates weekly(default is 'yes')?"
                echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
                if [ "$USE_DEFAULTS" == "NO" ]; then
                    read -r YES_NO
                else
                    YES_NO=""
                fi
                if [ ! -e ~/.ImpactControlCenter ]; then
                    touch ~/.ImpactControlCenter
                fi
                if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                    if [ "$(grep -c AutoCheckForUpdatesWeekly ~/.ImpactControlCenter)" -ne "0" ]; then
                        Tweakline=$(($(grep -n "AutoCheckForUpdatesWeekly" ~/.ImpactControlCenter | cut -d: -f1))) && sed -i "$Tweakline s/.*/AutoCheckForUpdatesWeekly=0/" ~/.ImpactControlCenter
                    else
                        echo "AutoCheckForUpdatesWeekly=0" >>~/.ImpactControlCenter
                    fi
                else
                    if [ "$(grep -c AutoCheckForUpdatesWeekly ~/.ImpactControlCenter)" -ne "0" ]; then
                        Tweakline=$(($(grep -n "AutoCheckForUpdatesWeekly" ~/.ImpactControlCenter | cut -d: -f1))) && sed -i "$Tweakline s/.*/AutoCheckForUpdatesWeekly=1/" ~/.ImpactControlCenter
                    else
                        echo "[MainFrame/Help]" >>~/.ImpactControlCenter
                        echo "AutoCheckForUpdatesWeekly=1" >>~/.ImpactControlCenter
                    fi
                fi
            fi
        fi
    fi
fi

if [ "$VDEV_SUPPORT" == "FALSE" ]; then
    rm -f "$DEF_DIRECTORY"/lib/"$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')"/libmvVirtualDevice.*
fi

if [ "$USB2_SUPPORT" == "TRUE" ]; then
    echo
    echo "Do you want to copy 51-mvbf.rules to /etc/udev/rules.d for non-root user support (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        echo
        echo "To grant non-root user support,"
        echo "copy 51-mvbf.rules the file to /etc/udev/rules.d"
        echo
    else
        $SUDO cp -f "$DEF_DIRECTORY/Scripts/51-mvbf.rules" /etc/udev/rules.d
    fi
fi

# copy the mvBF3 boot-device and an universal udev rules file for U3V cameras to the system
if [ "$U3V_SUPPORT" == "TRUE" ]; then
    echo
    echo "Do you want to copy the necessary files to /etc/udev/rules.d for U3V device support (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        echo
        echo 'To be able to use U3V devices, copy 52-U3V.rules and 52-mvbf3.rules the file to /etc/udev/rules.d'
        echo
    else
        $SUDO cp -f "$DEF_DIRECTORY/Scripts/52-U3V.rules" /etc/udev/rules.d
        $SUDO cp -f "$DEF_DIRECTORY/Scripts/52-mvbf3.rules" /etc/udev/rules.d
    fi
fi

# install the BVS CA-BN (mvBlueNAOS) universal udev rules file and scripts
if [ "$PCIE_SUPPORT" == "TRUE" ]; then
    echo
    echo "Do you want to copy the necessary files to /etc/udev/rules.d for BVS CA-BN (mvBlueNAOS) PCIe device support (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        echo
        echo 'To be able to use PCIe devices, copy 51-udev-pcie.rules to /etc/udev/rules.d'
        echo 'and make_device.sh and device_namer.sh to /usr/local/bin/.'
    else
        $SUDO cp -f "$DEF_DIRECTORY/Scripts/51-udev-pcie.rules" /etc/udev/rules.d
        $SUDO cp -f "$DEF_DIRECTORY/Scripts/make_device.sh" /usr/local/bin/
        $SUDO cp -f "$DEF_DIRECTORY/Scripts/device_namer.sh" /usr/local/bin/
    fi
fi
echo

# reload the udev rules after change.
$SUDO "$UDEVADM" control --reload-rules

# building kernel module for BVS CA-BN (mvBlueNAOS)
if [ "$PCIE_SUPPORT" == "TRUE" ]; then
    echo
    echo "Do you want the BVS CA-BN (mvBlueNAOS) PCIe kernel module to be built (default is 'yes')?"
    echo "The kernel module MUST be built for PCIe cameras to be usable!"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        echo
        echo "Please restart the installer and select 'YES' if you want to use a PCIe camera!"
        ((ERROR_NUMBER = ERROR_NUMBER + 1))
        echo
    else
        if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ]; then
            if [ -n "$JETSON_KERNEL" ]; then
                HEADERS_PKG=nvidia-l4t-kernel-headers
            elif [ -f /proc/device-tree/model ] && [ "$(grep -i 'Raspberry' /proc/device-tree/model)" -ne 1 ]; then
                HEADERS_PKG=raspberrypi-kernel-headers
            else
                HEADERS_PKG=linux-headers-$KERNEL_VERSION
            fi
            REQUIRED_PACKAGES=("$HEADERS_PKG")
            installPackages "${REQUIRED_PACKAGES[@]}"
        elif [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
            REQUIRED_PACKAGES=(kernel-devel)
            installPackages "${REQUIRED_PACKAGES[@]}"
        elif [ "$OS_NAME" == "RedHat" ] || [ "$OS_NAME" == "Fedora" ] || [ "$OS_NAME" == "CentOS" ]; then
            REQUIRED_PACKAGES=(kernel-devel)
            installPackages "${REQUIRED_PACKAGES[@]}"
        else
            distroNotSupportedMessage "${REQUIRED_PACKAGES[@]}"
        fi
    fi

    if [ -r "$DEF_DIRECTORY" ]; then
        BN_INSTALLED="FALSE"
        KM_SRC_DIR=$DEF_DIRECTORY/kernelmodules/linux/PCIe
        BN_MODNAME="mvpci"
        DKMS=$($SUDO which dkms)

        # unload any older version of the kernel module
        MOD_STATUS=$(lsmod | grep -w "$BN_MODNAME")
        if [ "x${MOD_STATUS}" != "x" ]; then
            $SUDO rmmod "$BN_MODNAME"
        fi
        # delete any older version of the kernel module that may have been installed without dkms
        $SUDO rm -f /lib/modules/"$(uname -r)"/kernel/misc/${BN_MODNAME}.ko

        # try to install using dkms if it is available
        if [ "x${DKMS}" != "x" ]; then
            BN_VERSION=$(grep "PACKAGE_VERSION" "$KM_SRC_DIR/dkms.conf" | awk -F '=' '{print $2}')
            # if exactly this version is registered with dkms then remove it
            STATUS=$($DKMS status "$BN_MODNAME/$BN_VERSION" | grep "installed")
            if [ "x${STATUS}" != "x" ]; then
                echo "The kernel module $BN_MODNAME/$BN_VERSION was already installed using dkms. Updating and rebuilding it for the current kernel."
                $SUDO "$DKMS" remove "$BN_MODNAME/$BN_VERSION" -k "$(uname -r)" -q
            fi

            echo "Adding, building and installing the Kernel module $BN_MODNAME/$BN_VERSION using dkms."
            $SUDO "$DKMS" install "$BN_MODNAME/$BN_VERSION"
            STATUS=$("$DKMS" status "$BN_MODNAME/$BN_VERSION" | grep "installed")
            if [ "x${STATUS}" != "x" ]; then
                echo "Kernel module $BN_MODNAME/$BN_VERSION installed using dkms. It will be automatically rebuilt if you update the kernel."
                BN_INSTALLED="TRUE"
            fi
        else
            echo "dkms not installed on this system! You will have to rebuild the kernel module $BN_MODNAME if you update the kernel."
        fi

        if [ $BN_INSTALLED != "TRUE" ]; then
            # fall back to compiling manually if dkms not possible
            echo "**********************************************************"
            echo ""
            echo "          BUILDING BVS CA-BN (mvBlueNAOS) kernel module..."
            echo ""
            echo "**********************************************************"
            cd "$KM_SRC_DIR" || exit 1

            if $SUDO make install; then
                BN_INSTALLED="TRUE"
            else
                ((ERROR_NUMBER = ERROR_NUMBER + 1))
            fi
        fi
    fi

    # load the new kernel module
    if [ "$BN_INSTALLED" = "TRUE" ]; then
        $SUDO modprobe "$BN_MODNAME"
    else
        echo "ERROR! The BVS CA-BN (mvBlueNAOS) kernel module could not be installed."
        exit 1
    fi
fi

# Update the library cache again.
$SUDO /sbin/ldconfig

# check if plugdev group exists and the user is member of it
if [ "$(grep -c ^plugdev: /etc/group)" == "0" ]; then
    echo "Group 'plugdev' doesn't exist, this is necessary to use USB devices as a normal user!"
    echo "Do you want to create it and add current user to 'plugdev' (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read -r YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        echo
        echo "'plugdev' will not be created and you can't run the device as non-root user!"
        echo "If you want non-root user support, you will need to create 'plugdev'"
        echo "and add these users to this group."
        ((WARNING_NUMBER = WARNING_NUMBER + 1))
    else
        $SUDO /usr/sbin/groupadd -g 46 plugdev
        $SUDO /usr/sbin/usermod -a -G plugdev "$USER"
        echo "Group 'plugdev' created and user '$USER' added to it."
    fi
else
    if [ "$(groups | grep -c plugdev)" == "0" ]; then
        echo "Group 'plugdev' exists, however user '$USER' is not a member, which is necessary to"
        echo "use USB devices. Do you want to add user '$USER' to 'plugdev' (default is 'yes')?"
        echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
        if [ "$USE_DEFAULTS" == "NO" ]; then
            read -r YES_NO
        else
            YES_NO=""
        fi
        if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
            echo
            echo "If you want to use USB devices you have to manually add user '$USER' to the plugdev group."
            ((WARNING_NUMBER = WARNING_NUMBER + 1))
        else
            $SUDO /usr/sbin/usermod -a -G plugdev "$USER"
            echo "User '$USER' added to 'plugdev' group."
        fi
    fi
fi
echo

# create the logs directory and set MVIMPACT_ACQUIRE_DATA_DIR.
if ! [ -d "$DEF_DATA_DIRECTORY/logs" ]; then
    mkdir -p "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
    if ! [ -d "$DEF_DATA_DIRECTORY/logs" ]; then
        # that didn't work, now try it as superuser
        $SUDO mkdir -p "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
    fi
fi

if [ "$GENICAM_SUPPORT_REQUIRED" == "TRUE" ]; then
    # create the cache for genicam xml files.
    if ! [ -d "$DEF_DATA_DIRECTORY/genicam" ]; then
        mkdir -p "$DEF_DATA_DIRECTORY/genicam" >/dev/null 2>&1
        if ! [ -d "$DEF_DATA_DIRECTORY/genicam" ]; then
            # that didn't work, now try it as superuser
            $SUDO mkdir -p "$DEF_DATA_DIRECTORY/genicam" >/dev/null 2>&1
            STATUS=$?
        fi
        if ! [ -d "$DEF_DATA_DIRECTORY/genicam" ]; then
            echo "ERROR: Could not create the '$DEF_DATA_DIRECTORY/genicam' directory."
            echo "Problem: $STATUS"
            echo "Maybe you specified a partition that was mounted read only?"
            echo
            exit
        fi
        $SUDO chmod 777 "$DEF_DATA_DIRECTORY/genicam"
    fi
fi

# make sure the complete Impact Acquire tree and the data folder belongs to the user
$SUDO chown -R "$USER": "$DEF_DIRECTORY"
$SUDO chown -R "$USER": "$DEF_DATA_DIRECTORY"

if [ "$RUNTIME_INSTALLATION" == "YES" ]; then
    # Create the ignoredInterfaces.txt file taking settings persistency into account
    if [ -r "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt" ]; then
        rm -f "$DEF_DIRECTORY/ignoredInterfaces.txt" >/dev/null 2>&1
    else
        mv "$DEF_DIRECTORY/ignoredInterfaces.txt" "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
        if ! [ -r "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt" ]; then
            $SUDO mv "$DEF_DIRECTORY/ignoredInterfaces.txt" "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
        fi
    fi
fi

# Configure the ignoredInterfaces.txt file according to the user preferences
if [ -r "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt" ]; then
    if [ "$GEV_SUPPORT" == "TRUE" ]; then
        sed -i '/GEV=I/d' "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt"
    else
        if [ "$(grep -c 'GEV=' "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt")" == "0" ]; then
            echo "GEV=I" >>"$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt"
        else
            sed -i "s/GEV=./GEV=I/" "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt"
        fi
    fi
    if [ "$U3V_SUPPORT" == "TRUE" ]; then
        sed -i '/U3V=I/d' "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt"
    else
        if [ "$(grep -c 'U3V=' "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt")" == "0" ]; then
            echo "U3V=I" >>"$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt"
        else
            sed -i "s/U3V=./U3V=I/" "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.txt"
        fi
    fi
fi

# Create the ignoredInterfaces.pcie.txt file taking settings persistency into account
if [ -r "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.pcie.txt" ]; then
    rm -f "$DEF_DIRECTORY/ignoredInterfaces.pcie.txt" >/dev/null 2>&1
else
    mv "$DEF_DIRECTORY/ignoredInterfaces.pcie.txt" "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
    if ! [ -r "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.pcie.txt" ]; then
        $SUDO mv "$DEF_DIRECTORY/ignoredInterfaces.pcie.txt" "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
    fi
fi

# Configure the ignoredInterfaces.pcie.txt file according to the user preferences
if [ -r "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.pcie.txt" ]; then
    if [ "$PCIE_SUPPORT" == "TRUE" ]; then
        sed -i '/PCI=I/d' "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.pcie.txt"
    else
        if [ "$(grep -c 'PCI=' "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.pcie.txt")" == "0" ]; then
            echo "PCI=I" >>"$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.pcie.txt"
        else
            sed -i "s/PCI=./PCI=I/" "$DEF_DATA_DIRECTORY/logs/ignoredInterfaces.pcie.txt"
        fi
    fi
fi

# Remove the deprecated directory 'mvBlueNAOSData' and its contents if it exists
if [ -d "$DEF_DATA_DIRECTORY/mvBlueNAOSData" ]; then
    rm -f "$DEF_DATA_DIRECTORY/mvBlueNAOSData"
fi

if [ -d "$DEF_DATA_DIRECTORY/logs" ]; then
    # exec (1) and write (2) are needed to create new files and reading (4) should be allowed anyway
    # therefore the permissions are set to 777, so any user is able to read and write logs
    $SUDO chmod 777 "$DEF_DATA_DIRECTORY/logs"
    mv "$DEF_DIRECTORY/apps/mvDebugFlags.mvd" "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
    if ! [ -r "$DEF_DATA_DIRECTORY/logs/mvDebugFlags.mvd" ]; then
        $SUDO mv "$DEF_DIRECTORY/apps/mvDebugFlags.mvd" "$DEF_DATA_DIRECTORY/logs" >/dev/null 2>&1
    fi
    if grep -q "MVIMPACT_ACQUIRE_DATA_DIR=" "$ACQUIRE_EXPORT_FILE"; then
        echo "MVIMPACT_ACQUIRE_DATA_DIR already defined in" "$ACQUIRE_EXPORT_FILE".
    else
        $SUDO sh -c "echo 'export MVIMPACT_ACQUIRE_DATA_DIR=$DEF_DATA_DIRECTORY' >> $ACQUIRE_EXPORT_FILE"
        STATUS=$?
    fi
else
    echo "ERROR: Could not create $DEF_DATA_DIRECTORY/logs directory."
    echo "Problem: $STATUS"
    echo "Maybe you specified a partition that was mounted read only?"
    echo
    exit 1
fi

# Configure the /etc/security/limits.d/acquire.conf file to be able to set thread priorities
if [ -d /etc/security/limits.d ]; then
    $SUDO touch /etc/security/limits.d/acquire.conf
    $SUDO chmod a+r /etc/security/limits.d/acquire.conf
    if [ "$(grep -c '@plugdev            -       nice            -20' /etc/security/limits.d/acquire.conf)" == "0" ]; then
        echo '@plugdev            -       nice            -20' | $SUDO tee -a /etc/security/limits.d/acquire.conf >/dev/null
    fi
    if [ "$(grep -c '@plugdev            -       rtprio          99' /etc/security/limits.d/acquire.conf)" == "0" ]; then
        echo '@plugdev            -       rtprio          99' | $SUDO tee -a /etc/security/limits.d/acquire.conf >/dev/null
    fi
else
    echo 'INFO: Directory /etc/security/limits.d is missing, Impact Acquire will not'
    echo 'be able to set thread priorities correctly. Incomplete frames may occur!!!'
    ((WARNING_NUMBER = WARNING_NUMBER + 1))
fi

#Set the necessary cti softlinks
if [ "$GEV_SUPPORT" == "TRUE" ] || [ "$U3V_SUPPORT" == "TRUE" ]; then
    createSoftlink "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED" libmvGenTLProducer.so mvGenTLProducer.cti
fi
if [ "$PCIE_SUPPORT" == "TRUE" ]; then
    createSoftlink "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED" libmvGenTLProducer.PCIe.so mvGenTLProducer.PCIe.cti
fi

if [[ ("$MINIMAL_INSTALLATION" == "NO" || "$RUNTIME_INSTALLATION" == "YES") ]]; then
    # A bunch of actions necessary if GEV support is enabled
    if [ "$GEV_SUPPORT" == "TRUE" ]; then
        # Ensure the necessary arping capabilities are set.
        $SUDO setcap cap_net_raw+ep "$(which arping)" >/dev/null 2>&1

        # Ensure the necessary capabilities for our applications are set.
        $SUDO setcap cap_net_bind_service,cap_net_raw+ep "$DEF_DIRECTORY/apps/ImpactControlCenter/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/ImpactControlCenter"
        $SUDO setcap cap_net_bind_service,cap_net_raw+ep "$DEF_DIRECTORY/apps/IPConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/IPConfigure"
        $SUDO setcap cap_net_bind_service,cap_net_raw+ep "$DEF_DIRECTORY/apps/DeviceConfigure/$(echo "$TARGET" | tr '[:upper:]' '[:lower:]')/DeviceConfigure"

        # Increase the network buffers to prevent incomplete frames and fine-tune reverse-path-filtering to allow for discovery of GEV cameras with bad network configurations.
        {
            echo 'net.core.wmem_max=4194304'
            echo 'net.core.rmem_max=16777216'
            echo 'net.core.wmem_default=4194304'
            echo 'net.core.rmem_default=16777216'
            echo 'net.core.udp_mem=10240 87380 16777216'
            echo 'net.core.netdev_max_backlog=10000'
            echo 'net.ipv4.conf.all.rp_filter = 2'
            echo 'net.ipv4.conf.default.rp_filter = 2'
        } >/tmp/62-buffers-performance.conf

        $SUDO mv /tmp/62-buffers-performance.conf /etc/sysctl.d/
        $SUDO sysctl -p /etc/sysctl.d/62-buffers-performance.conf >/dev/null 2>&1
    fi

    # Check if a systemd service should handle usbcore.usbfs_memory_mb settings
    if [ "$U3V_SUPPORT" == "TRUE" ]; then
        if [ "$INIT_SYSTEM" == "systemd" ]; then
            echo
            echo "Since this system uses systemd as init-system, a systemd service could be created now that would automatically"
            echo "invoke the settings from file 'usbfs_memory_mb' at boot time which would then be determined by this script."
            echo
            echo "The size of this memory corresponds to the amount of memory that can be used by all USB3 Vision cameras as"
            echo "well as all other USB devices in the system. Usually USB3 Vision devices use most if not all of this memory"
            echo "for queuing capture buffers thus more memory will allow more buffers/cameras to be used in parallel!"
            echo
            echo "For more details regarding usbcore.usbfs_memory_mb settings, including sample calculations, please visit:"
            echo "https://www.balluff.com/en-de/online-manuals-mv/mvBlueFOX3/index.html"
            echo " -> Quickstart"
            echo "  -> Optimizing USB Performance"
            echo "   -> Checklist for Linux"
            echo "    -> Increasing Kernel memory"
            echo
            echo "Would you like to proceed like this (default is 'yes')?"
            echo
            echo "Selecting 'yes' will cause this script to create a systemd-service which will configure the usbfs_memory_mb"
            echo "parameter to make sure the USB3 Vision cameras work as expected."
            echo
            echo "Selecting 'no' will make it necessary to configure the value manually as it is mandatory for USB3 Vision"
            echo "devices to work without image transfer issues."
            echo
            echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
            if [ "$USE_DEFAULTS" == "NO" ]; then
                read -r YES_NO
            else
                YES_NO=""
            fi
            if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                SYSTEMS_USBFS_MEMORY_MB_VALUE=$(</sys/module/usbcore/parameters/usbfs_memory_mb)
                SYSTEMS_MEMORY_TOTAL_KB=$(grep "MemTotal:" /proc/meminfo | sed 's/[^0-9]*//g')
                ((SYSTEMS_MEMORY_TOTAL_MB = SYSTEMS_MEMORY_TOTAL_KB / 1024))
                echo
                echo "How much memory should be reserved for USB traffic (OS default is typically 16MB which"
                echo "is by far not enough to operate USB3 Vision cameras with high bandwidth usage. A value"
                echo "of around 256MB is usually a better choice)?"
                echo
                echo "It will be important to select a good compromise between the amount of data which is expected"
                echo "and the total memory which is available."
                echo
                echo "Insufficient memory will lead to image transmission issues and in some cases even to issues"
                echo "when configuring devices!"
                echo
                echo "Too much memory might cause a huge amount of unused memory reserved for USB traffic that could"
                echo "otherwise be used by the operating system for other tasks."
                echo
                echo "PROVIDING MORE MEMORY THAN NECESSARY DOES NOT HAVE ANY POSITIVE IMPACT!"
                echo
                echo "For more details regarding usbcore.usbfs_memory_mb settings, including sample calculations, please visit:"
                echo "https://www.balluff.com/en-de/online-manuals-mv/mvBlueFOX3/index.html"
                echo " -> Quickstart"
                echo "  -> Optimizing USB Performance"
                echo "   -> Checklist for Linux"
                echo "    -> Increasing Kernel memory"
                echo
                echo "Currently configured: ${SYSTEMS_USBFS_MEMORY_MB_VALUE} MB"
                echo "Total available RAM:  ${SYSTEMS_MEMORY_TOTAL_MB} MB"
                echo
                echo "Please specify the amount of memory (in Megabyte) which should be reserved for USB data"
                read -r USER_SELECTED_MEMORY
                STATUS=$?
                if [ "$SYSTEMS_MEMORY_TOTAL_MB" -gt "$USER_SELECTED_MEMORY" ]; then
                    USBFS_MEMORY_MB_VALUE=$USER_SELECTED_MEMORY
                elif [ "$STATUS" -ne 0 ]; then
                    USBFS_MEMORY_MB_VALUE=256
                    echo "Error! Invalid value passed. Will use default value: ${USBFS_MEMORY_MB_VALUE} MB"
                else
                    USBFS_MEMORY_MB_VALUE=256
                    echo "Error! Specified memory is more than the system's total memory!"
                    echo "Configuring to the default value of: ${USBFS_MEMORY_MB_VALUE} MB"
                fi
            else
                USBFS_MEMORY_MB_VALUE=256
            fi
            echo "Configuring usbcore.usbfs_memory_mb automatically to use: ${USBFS_MEMORY_MB_VALUE} MB for"
            echo "USB traffic"
            USBFS_CONFIGURED_PER_SYSTEMD="YES"
            echo -e "#!/bin/bash
USBFS_MEMORY_MB_VALUE=$USBFS_MEMORY_MB_VALUE
SYSTEMS_USBFS_MEMORY_MB_VALUE=\$(cat /sys/module/usbcore/parameters/usbfs_memory_mb)
echo \"usbcore.usbfs_memory_mb:  \${SYSTEMS_USBFS_MEMORY_MB_VALUE}\" | systemd-cat -p info
echo \"Changing to: \${USBFS_MEMORY_MB_VALUE} MB\"
sh -c \"echo \${USBFS_MEMORY_MB_VALUE} > /sys/module/usbcore/parameters/usbfs_memory_mb\"

SYSTEMS_USBFS_MEMORY_MB_VALUE=\$(cat /sys/module/usbcore/parameters/usbfs_memory_mb)
if [ $? -eq 0 ]; then
echo \"usbcore.usbfs_memory_mb: Changed sucessfully to: \${SYSTEMS_USBFS_MEMORY_MB_VALUE} MB\" | systemd-cat -p info
else
echo \"usbcore.usbfs_memory_mb: Could not be modified. Error: ${?}\" | systemd-cat -p error
fi
exit 0" >"$DEF_DIRECTORY/Scripts/resize_usbfs_buffersize.sh"

            $SUDO chmod +x "$DEF_DIRECTORY/Scripts/resize_usbfs_buffersize.sh"
            echo -e "
[Unit]
Description=Balluff - USB buffer size modification service

Wants=network.target
After=syslog.target network-online.target

[Service]
Type=simple
ExecStart=$DEF_DIRECTORY/Scripts/resize_usbfs_buffersize.sh
Restart=on-failure
RestartSec=10
KillMode=process

[Install]
WantedBy=multi-user.target
" >/tmp/resize_usbfs_buffersize.service
            $SUDO mv /tmp/resize_usbfs_buffersize.service /etc/systemd/system
            $SUDO systemctl enable resize_usbfs_buffersize.service  >/dev/null 2>&1
            $SUDO systemctl start resize_usbfs_buffersize.service  >/dev/null 2>&1
        else
            echo "Error: Init system does not support usbfs-modifications!"
        fi
    fi

    # Check whether the network buffers are configured
    if [ "$GEV_SUPPORT" == "TRUE" ]; then
        ERROR=0
        echo "------------------------------------GEV Check--------------------------------------"
        if [ "$(which sysctl)" == "" ]; then
            echo "Warning: 'sysctl' not present on the system, network parameters cannot be checked!"
            ERROR=1
        else
            RMEM=$(($(sysctl -n net.core.rmem_max) / 1048576))
            WMEM=$(($(sysctl -n net.core.wmem_max) / 1048576))
            BKLG=$(sysctl -n net.core.netdev_max_backlog)
            if [ "$RMEM" -lt 16 ]; then
                if [ "$RMEM" -lt 1 ]; then
                    echo "Warning: 'net.core.rmem_max' Receive buffer settings are low( less than 1MB )!"
                else
                    echo "Warning: 'net.core.rmem_max' Receive buffer settings are low($RMEM MB)!"
                fi
                ERROR=1
            fi
            if [ "$WMEM" -lt 4 ]; then
                if [ "$WMEM" -lt 1 ]; then
                    echo "Warning: 'net.core.rmem_max' Transmit buffer settings are low( less than 1MB )!"
                else
                    echo "Warning: 'net.core.rmem_max' Transmit buffer settings are low($WMEM MB)!"
                fi
                ERROR=1
            fi
            if [ "$BKLG" -lt 10000 ]; then
                echo "Warning: 'net.core.netdev_max_backlog' input queue settings are low($BKLG elements)!"
                ERROR=1
            fi
            if [ $ERROR == 1 ]; then
                echo "Not all network parameters are optimized. Incomplete frames may occur during image acquisition!"
            fi
        fi
        if [ $ERROR == 1 ]; then
            ((WARNING_NUMBER = WARNING_NUMBER + 1))
            echo
            echo "Please refer to 'Quickstart/Optimizing the network configuration' section of the "
            echo "User Manual for more information on how to adjust the network buffers"
            echo "https://www.balluff.com/en-de/online-manuals-mv/mvBlueCOUGAR-X/mvBC_page_quickstart.html#mvBC_subsubsection_quickstart_network_configuration_controller"
            echo "-----------------------------------------------------------------------------------"
        else
            echo "${green}${bold}                                       OK!${reset}                  "
            echo "-----------------------------------------------------------------------------------"
        fi
    fi

    # Check whether the USBFS Memory is configured
    if [ "$U3V_SUPPORT" == "TRUE" ]; then
        ERROR=0
        echo "------------------------------------U3V Check--------------------------------------"
        if [ ! -r /sys/module/usbcore/parameters/usbfs_memory_mb ]; then
            echo "Warning: 'usbfs_memory_mb' parameter does not exist or cannot be read!"
            ERROR=1
        else
            USBMEM=$(cat /sys/module/usbcore/parameters/usbfs_memory_mb)
            if [ "$USBMEM" -lt 256 ] && [ ! "$USBFS_CONFIGURED_PER_SYSTEMD" = "YES" ]; then
                echo "Warning: 'usbfs_memory_mb' Kernel USB file system buffer settings are low($USBMEM MB)!"
                echo "Incomplete frames may occur during image acquisition!"
                ERROR=1
            fi
        fi
        if [ $ERROR == 1 ]; then
            ((WARNING_NUMBER = WARNING_NUMBER + 1))
            echo
            echo "Please refer to 'Quickstart/Linux/Optimizing USB performance' section of the "
            echo "User Manual for more information on how to adjust the kernel USB buffers"
            echo
            echo "https://www.balluff.com/en-de/online-manuals-mv/mvBlueFOX3/index.html"
            echo " -> Quickstart"
            echo "  -> Optimizing USB Performance"
            echo "   -> Checklist for Linux"
            echo "    -> Increasing Kernel memory"
            echo
            echo "-----------------------------------------------------------------------------------"
        else
            echo "${green}${bold}                                       OK!${reset}                  "
            echo "-----------------------------------------------------------------------------------"
        fi
    fi
fi

# remove all example application sources in case of minimal installation
if [ "$MINIMAL_INSTALLATION" == "YES" ] || [ "$RUNTIME_INSTALLATION" == "YES" ]; then
    $SUDO rm -rf "$DEF_DIRECTORY"/apps >/dev/null 2>&1
fi

# resetting wxWidgets configuration to the auto default
if [ "$WXCONFIG_RESET_NEEDED" ]; then
    echo "Resetting wxWidgets configuration to the auto default."
    $SUDO update-alternatives --auto wx-config
fi

if [ "$GENICAM_SUPPORT_REQUIRED" == "TRUE" ]; then
    if [ -r $GENICAM_EXPORT_FILE ]; then
        echo
        # Shellcheck should not report an SC1090 issue as the called script is created by the script itself.
        # shellcheck source=/dev/null
        . "$GENICAM_EXPORT_FILE"
        echo
    else
        echo "It seems as if the GenICam Export file ($GENICAM_EXPORT_FILE) could not be read!"
        echo "This might cause GenICam devices to be not usable."
        ((WARNING_NUMBER = WARNING_NUMBER + 1))
    fi
fi

echo
if [ "$ERROR_NUMBER" == 0 ] && [ "$WARNING_NUMBER" == 0 ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${green}${bold}                           Installation successful!${reset}         "
    echo "-----------------------------------------------------------------------------------"
elif [ "$ERROR_NUMBER" == 0 ] && [ "$WARNING_NUMBER" != 0 ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${yellow}${bold}                           Installation successful!${reset}        "
    echo "-----------------------------------------------------------------------------------"
    echo "                                                                                   "
    echo "  Some warnings have been issued during the installation. Typically the driver     "
    echo "  will work, but some functionalities are missing e.g. some sample applications    "
    echo "  which could not be built because of missing dependencies or not optimized NIC-   "
    echo "  settings.                                                                        "
    echo "                                                                                   "
    echo "  Please refer to the output of the script for further details.                    "
    echo "-----------------------------------------------------------------------------------"
else
    echo "-----------------------------------------------------------------------------------"
    echo "${red}${bold}                        Installation NOT successful!${reset}          "
    echo "-----------------------------------------------------------------------------------"
    echo "                                                                                   "
    echo "  Please provide the full output of this installation script to the Balluff "
    echo "  support department (support.de@balluff.de) if the error messages shown during   "
    echo "  the installation procedure don't help you to get the driver package installed    "
    echo "  correctly!                                                                       "
    echo "-----------------------------------------------------------------------------------"
fi
echo
echo "Do you want to reboot now (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$USE_DEFAULTS" == "NO" ]; then
    read -r YES_NO
else
    YES_NO="n"
fi
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    echo "You need to reboot manually to complete the installation."
else
    $SUDO shutdown -r now
fi
